package com.idea.customerManagement.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.excel.MnpBuildingExcel;
import com.idea.buildManage.excel.ParkRoomExcel;
import com.idea.buildManage.excel.WriteHandlerStrategy;
import com.idea.buildManage.mapper.ParkFloorDiscMapper;
import com.idea.buildManage.mapper.ParkInfoMapper;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.ParkFloorDisc;
import com.idea.buildManage.model.ParkInfo;
import com.idea.buildManage.model.ParkInfoExample;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.buildManage.service.ParkFloorDiscService;
import com.idea.buildManage.service.ParkInfoService;
import com.idea.customerManagement.dto.CustomerManagementDto;
import com.idea.customerManagement.dto.RoomSelectionInfoDto;
import com.idea.customerManagement.excel.CustomerManagementExcel;
import com.idea.customerManagement.mapper.CustomerManagementExtendMapper;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.mapper.RoomSelectionInfoMapper;
import com.idea.customerManagement.model.*;
import com.idea.invoice.util.InvoiceUtil;
import com.idea.util.DateUtils;
import com.idea.util.ExcelUtils;
import com.idea.util.ReplaceWord;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class CustomerManagementService implements BaseService<CustomerManagement, CustomerManagementExample> {


    @Autowired
    private CustomerManagementMapper modelMapper;
    @Autowired
    private CustomerManagementExtendMapper extendMapper;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private SysDictService sysDictService;
    @Autowired
    private FrameUserService frameUserService;
    @Autowired
    private RoomSelectionInfoMapper roomSelectionInfoMapper;
    @Autowired
    private ParkRoomMapper parkRoomMapper;
    @Autowired
    private ParkInfoMapper parkInfoMapper;
    @Autowired
    private ParkFloorDiscService floorDiscService;
    @Autowired
    private ParkInfoService parkInfoService;
    @Autowired
    private IntentionalDepositService intentionalDepositService;
    @Autowired
    private ParkFloorDiscMapper parkFloorDiscMapper;

    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(CustomerManagement customerManagement) {
        customerManagement.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(customerManagement);
    }

    public int insertWithOutId(CustomerManagement customerManagement) {
        return modelMapper.insert(customerManagement);
    }

    @Override
    public CustomerManagement selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(CustomerManagement customerManagement) {
        return modelMapper.updateByPrimaryKeySelective(customerManagement);
    }

    @Override
    public int updateByExampleSelective(CustomerManagement customerManagement, CustomerManagementExample customerManagementExample) {
        return modelMapper.updateByExampleSelective(customerManagement, customerManagementExample);
    }

    @Override
    public int updateByExample(CustomerManagement customerManagement, CustomerManagementExample customerManagementExample) {
        return modelMapper.updateByExample(customerManagement, customerManagementExample);
    }

    @Override
    public List<CustomerManagement> selectByExample(CustomerManagementExample customerManagementExample) {
        return modelMapper.selectByExample(customerManagementExample);
    }

    @Override
    public long countByExample(CustomerManagementExample customerManagementExample) {
        return modelMapper.countByExample(customerManagementExample);
    }

    @Override
    public int deleteByExample(CustomerManagementExample customerManagementExample) {
        return modelMapper.deleteByExample(customerManagementExample);
    }

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<CustomerManagementDto> listByModel(Tablepar tablepar, CustomerManagementDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        if (StringUtils.isNotEmpty(model.getGroupIds())) {
            model.setGroupIdList(Arrays.asList(model.getGroupIds().split(",")));
        }
        if (StringUtils.isNotEmpty(model.getDiscIds())) {
            model.setDiscIdList(Arrays.asList(model.getDiscIds().split(",")));
        }
        List<CustomerManagementDto> list = extendMapper.listByModel(model);
        PageInfo<CustomerManagementDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    public List<CustomerManagementDto> listAll(CustomerManagementDto model, DateTrans dt) {
        if (StringUtils.isNotEmpty(model.getGroupIds())) {
            model.setGroupIdList(Arrays.asList(model.getGroupIds().split(",")));
        }
        if (StringUtils.isNotEmpty(model.getDiscIds())) {
            model.setDiscIdList(Arrays.asList(model.getDiscIds().split(",")));
        }
        List<CustomerManagementDto> list = extendMapper.listByModel(model);
        return list;
    }


    /**
     * 新增轮号客户
     *
     * @param model
     * @return
     */
    public int add(CustomerManagementDto model) {

        CustomerManagementExample example = new CustomerManagementExample();
        example.createCriteria().andBatchNumberEqualTo(model.getBatchNumber())
                .andRoomSelectionNumberEqualTo(model.getRoomSelectionNumber());
        long count = countByExample(example);
        if (count > 0) {
            throw new RuntimeException("批次号选房号重复!!!");
        }

        String buyerJson = model.getBuyerJson();
        String id = IdUtil.simpleUUID();
        String userId = ShiroUtils.getUserId();
        Date date = new Date();
        List<Buyer> buyers = JSONObject.parseArray(buyerJson, Buyer.class);
        int i = 1;
        StringBuilder builder = new StringBuilder();
        StringBuilder identityCardBuilder = new StringBuilder();
        for (Buyer buyer : buyers) {
            buyer.setOrderNum(i);
            i++;
            buyer.setCustomerManagementId(id);
            buyer.setCreatedAt(date);
            buyer.setCreatedId(userId);
            buyerService.insertSelective(buyer);
            if (StringUtils.isEmpty(buyer.getRelationship())) {
                builder.append(buyer.getName()).append(",");
            } else {
                String dictLabel = sysDictService.getDictLabel("RELATIONSHIP", buyer.getRelationship());
                builder.append(buyer.getName())
                        .append("(")
                        .append(dictLabel)
                        .append(")")
                        .append(",");
            }
            identityCardBuilder.append(buyer.getIdentityCard()).append(",");
        }
        if (builder.length() > 0) {
            builder.deleteCharAt(builder.length() - 1);
        }
        if (identityCardBuilder.length() > 0) {
            identityCardBuilder.deleteCharAt(identityCardBuilder.length() - 1);
        }
        model.setBuyerName(builder.toString());
        model.setBuyerIdentityCard(identityCardBuilder.toString());
        model.setId(id);
        // 待选房
        model.setStatus(1);
        model.setCreatedAt(date);
        model.setCreatedId(userId);
        int result = insertWithOutId(model);

//        try {
//            // 给nc推送客户数据
//            JSONObject json = new JSONObject();
//            json.put("srcsystemid", id);
//            json.put("name", builder.toString());
//            json.put("pk_custclass", "02"); //客户基本分类 ，，（默认是02），必选，01内部客户，02外部客户
//            json.put("custprop", "0");//财务组织客户类型，必选，默认0
//            json.put("taxpayerid",identityCardBuilder.toString() ); //统一社会信用代码，必选
//            log.info("推送客户详情:{}", json.toJSONString());
//            String resultJson = InvoiceUtil.customerAdd(json);
//            JSONObject jsonObject = JSONObject.parseObject(resultJson);
//            String success = jsonObject.getString("success");
//            if (success.equals("true")) {
//                JSONObject data = jsonObject.getJSONObject("data");
//                String code = data.getString("code");
//                String ncid = data.getString("ncid");
//                CustomerManagement record = new CustomerManagement();
//                record.setId(id);
//                record.setNcid(ncid);
//                record.setNcCode(code);
//                updateByPrimaryKeySelective(record);
//            }
//        }catch (Exception e){
//            log.error("推送客户失败",e);
//        }

//        try {
//            // 客户分配
//            JSONObject json = new JSONObject();
//            json.put("pk_org", InvoiceUtil.pk_org);
//            json.put("ncid", identityCardBuilder.toString());
//            InvoiceUtil.customerAssign(json);
//        }catch (Exception e){
//            log.error("客户分配失败",e);
//        }

        return result;

    }


    /**
     * 编辑轮号客户
     *
     * @param model
     * @return
     */
    public int edit(CustomerManagementDto model) {

        String buyerJson = model.getBuyerJson();
        String id = model.getId();
        String userId = ShiroUtils.getUserId();
        Date date = new Date();

        CustomerManagement oldCustomer = selectByPrimaryKey(id);

        // 先删除
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(id);
        List<Buyer> oldBuyers = buyerService.selectByExample(buyerExample);
        for (Buyer buyer : oldBuyers) {
            buyerService.deleteByPrimaryKey(buyer.getId());
        }
        // 再新增
        List<Buyer> buyers = JSONObject.parseArray(buyerJson, Buyer.class);
        int i = 1;
        StringBuilder builder = new StringBuilder();
        StringBuilder identityCardBuilder = new StringBuilder();
        for (Buyer buyer : buyers) {
            buyer.setOrderNum(i);
            i++;
            buyer.setCustomerManagementId(id);
            buyer.setCreatedAt(date);
            buyer.setCreatedId(userId);
            buyerService.insertSelective(buyer);
            if (StringUtils.isEmpty(buyer.getRelationship())) {
                builder.append(buyer.getName()).append(",");
            } else {
                String dictLabel = sysDictService.getDictLabel("RELATIONSHIP", buyer.getRelationship());
                builder.append(buyer.getName())
                        .append("(")
                        .append(dictLabel)
                        .append(")")
                        .append(",");
            }
            identityCardBuilder.append(buyer.getIdentityCard()).append(",");
        }
        if (builder.length() > 0) {
            builder.deleteCharAt(builder.length() - 1);
        }
        if (identityCardBuilder.length() > 0) {
            identityCardBuilder.deleteCharAt(identityCardBuilder.length() - 1);
        }
        model.setBuyerName(builder.toString());
        model.setBuyerIdentityCard(identityCardBuilder.toString());
        model.setUpdatedAt(date);
        model.setUpdatedId(userId);

        int result = updateByPrimaryKeySelective(model);

//        try {
//
//            // 修改客户数据
//            JSONObject json = new JSONObject();
//            json.put("ncid", oldCustomer.getBuyerIdentityCard());// 客户统一信用代码，必选
//            json.put("name", builder.toString());
//            json.put("pk_custclass", "02"); //客户基本分类 ，，（默认是02），必选，01内部客户，02外部客户
//            json.put("custprop", "0");//财务组织客户类型，必选，默认0
//            json.put("taxpayerid", identityCardBuilder.toString()); //统一社会信用代码，必选
//            log.info("推送客户修改详情:{}", json.toJSONString());
//            String resultJson = InvoiceUtil.customerUpdate(json);
//            JSONObject jsonObject = JSONObject.parseObject(resultJson);
//            String success = jsonObject.getString("success");
//            if (success.equals("true")) {
//            }
//        }catch (Exception e){
//            log.error("修改客户失败",e);
//        }

        return result;

    }


    /**
     * 删除
     *
     * @param ids
     * @return
     */
    public int delete(String ids) {
        String[] idLsit = ids.split(",");
        for (String id : idLsit) {
            CustomerManagement customerManagement = selectByPrimaryKey(id);
            String houseId = customerManagement.getHouseId();
            if (StringUtils.isNotEmpty(houseId)) {
                throw new RuntimeException("该数据已在别的业务中引用，无法删除!!!");
            }
        }
        for (String id : idLsit) {
            deleteByPrimaryKey(id);
        }
        return 1;
    }

    /**
     * 查看详情
     *
     * @param id
     * @return
     */
    public CustomerManagementDto getById(String id) {
        CustomerManagement customerManagement = selectByPrimaryKey(id);
        CustomerManagementDto result = new CustomerManagementDto();
        BeanUtil.copyProperties(customerManagement, result);
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(id);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        if (CollectionUtils.isNotEmpty(buyers)) {
            String json = JSON.toJSON(buyers).toString();
            result.setBuyerJson(json);
        }
        if (StringUtils.isNotEmpty(customerManagement.getAbandonmentId())) {
            FrameUser frameUser = frameUserService.selectByPrimaryKey(customerManagement.getAbandonmentId());
            String name = frameUser == null ? "" : frameUser.getTruename();
            result.setAbandonmentName(name);
        }

        BigDecimal intentionalDepositMoney = BigDecimal.ZERO;
        // 默认意向金 未收取
        int intentionalDepositStatus = 0;
        IntentionalDepositExample intentionalDepositExample = new IntentionalDepositExample();
        intentionalDepositExample.createCriteria().andCustomerManagementIdEqualTo(id);
        List<IntentionalDeposit> intentionalDeposits = intentionalDepositService.selectByExample(intentionalDepositExample);
        if (CollectionUtils.isNotEmpty(intentionalDeposits)) {
            IntentionalDeposit intentionalDeposit = intentionalDeposits.get(0);
            intentionalDepositStatus = intentionalDeposit.getStatus();
            // 已收取
            if (intentionalDeposit.getStatus() == 1) {
                intentionalDepositMoney = intentionalDeposit.getReceivedAmount();
            }
        }
        result.setIntentionalDepositMoney(intentionalDepositMoney);
        result.setIntentionalDepositStatus(intentionalDepositStatus);


        RoomSelectionInfoExample roomSelectionInfoExample = new RoomSelectionInfoExample();
        roomSelectionInfoExample.createCriteria().andCustomerManagementIdEqualTo(id);
        long count = roomSelectionInfoMapper.countByExample(roomSelectionInfoExample);
        result.setRoomSelectionCount((int) count);

        if(StringUtils.isNotEmpty(customerManagement.getGroupId()) && StringUtils.isNotEmpty(customerManagement.getDiscId())){
            ParkInfo parkInfo = parkInfoMapper.selectByPrimaryKey(customerManagement.getGroupId());
            ParkFloorDisc parkFloorDisc = parkFloorDiscMapper.selectByPrimaryKey(customerManagement.getDiscId());
            result.setGroupDiscName(parkInfo.getGroupName() + parkFloorDisc.getName());
        }
        return result;
    }

    /**
     * 放弃资格
     *
     * @param model
     * @return
     */
    public int giveUp(CustomerManagement model) {

        model.setStatus(6);
        model.setAbandonmentAt(new Date());
        model.setAbandonmentId(ShiroUtils.getUserId());
        int result = updateByPrimaryKeySelective(model);

        RoomSelectionInfoExample example = new RoomSelectionInfoExample();
        example.createCriteria().andCustomerManagementIdEqualTo(model.getId());
        List<RoomSelectionInfo> list = roomSelectionInfoMapper.selectByExample(example);
        for (RoomSelectionInfo roomSelectionInfo : list) {
            ParkRoom parkRoom = new ParkRoom();
            parkRoom.setId(roomSelectionInfo.getHouseId());
            parkRoom.setIsChoose(0);
            parkRoomMapper.updateByPrimaryKeySelective(parkRoom);
        }

        return result;
    }

    /**
     * 选房通知单下载
     *
     * @param id
     * @param response
     */
    public void downLoad(String id, HttpServletResponse response) {

        String filePath = "/home/project/settleDown/tem/houseSelectionNotice.docx";
        String fileName = "选房通知单.docx";
        CustomerManagement customerManagement = selectByPrimaryKey(id);
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(id);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        if (CollectionUtils.isNotEmpty(buyers)) {
            Buyer buyer = buyers.get(0);
            String name = buyer.getName();
            String identityCard = buyer.getIdentityCard();
            String roomSelectionNumber = customerManagement.getRoomSelectionNumber();
            Date roomSelectionDate = customerManagement.getRoomSelectionDate();
            String year = DateUtils.parseDateToStr(DateUtils.YYYY, roomSelectionDate);
            String month = DateUtils.parseDateToStr(DateUtils.MM, roomSelectionDate);
            String date = DateUtils.parseDateToStr(DateUtils.DD, roomSelectionDate);
            ParkInfo parkInfo = parkInfoMapper.selectByPrimaryKey(customerManagement.getGroupId());
            String region = "";
            String dictLabel = sysDictService.getDictLabel("REGIONALISM_CODE", parkInfo.getRegionalismCode());
            if (StringUtils.isNotEmpty(dictLabel)) {
                region = dictLabel;
            }
            Map<String, Object> data = new HashMap<>();
            Map<String, Object> picData = new HashMap<>();
            Map<String, Object> sealData = new HashMap<>();
            Map<String, Object> settingData = new HashMap<>();
            data.put("${name}", name);
            data.put("${identityCard}", identityCard);
            data.put("${roomSelectionNumber}", roomSelectionNumber);
            data.put("${year}", year);
            data.put("${month}", month);
            data.put("${date}", date);
            data.put("${region}", region);
            ReplaceWord.operateWord(response, filePath, fileName, data, picData, sealData, settingData);
        }


    }

//    public JSONObject handleImport(MultipartFile file) throws IOException {
//
//        String userId = ShiroUtils.getUserId();
//        // 读取excel
//        ExcelUtils excelUtils = new ExcelUtils();
//        List<CustomerManagementExcel> list = excelUtils.excelImport(file.getInputStream(), CustomerManagementExcel.class);
//
//        Map<String, String> discNameIdMap = floorDiscService.getNameIdMap();
//        Map<String, String> parkNameIdMap = parkInfoService.getNameIdMap();
//        List<Map<String, String>> relationship = sysDictService.selectDictList("RELATIONSHIP");
//
//
//        List<CustomerManagementExcel> badList = new ArrayList<>();
//        int count;
//        JSONObject jsonObject = new JSONObject();
//
//        // 先校验
//        for(CustomerManagementExcel excel : list){
//
//            new CustomerManagement();
//
//            boolean groupNameFlag = true;
//            boolean discNameFlag = true;
//            boolean roomSelectionDateFlag = true;
//            boolean batchNumberFlag = true;
//            boolean roomSelectionNumberFlag = true;
//            boolean buyerNameFlag = true;
//            boolean identityCardFlag = true;
//            boolean phoneFlag = true;
//            boolean relationshipFlag = true;
//
//            StringBuilder builder = new StringBuilder();
//            try {
//
//                if (StringUtils.isEmpty(excel.getGroupName())) {
//                    groupNameFlag = false;
//                    builder.append("楼盘/小区名称为空").append(",");
//                    excel.setGroupName("{" + excel.getGroupName() + "}");
//                }
//
//                if (StringUtils.isEmpty(excel.getDiscName())) {
//                    discNameFlag = false;
//                    builder.append("楼分期名称为空").append(",");
//                    excel.setDiscName("{" + excel.getDiscName() + "}");
//                }
//
//                // 判断小区是否存在
//                if(StringUtils.isEmpty(parkNameIdMap.get(excel.getGroupName()))){
//                    groupNameFlag = false;
//                    builder.append("楼盘/小区不存在").append(",");
//                    excel.setGroupName("{" + excel.getGroupName() + "}");
//                }
//
//                // 判断分期是否存在
//                if(StringUtils.isEmpty(discNameIdMap.get(excel.getDiscName()))){
//                    discNameFlag = false;
//                    builder.append("分期不存在").append(",");
//                    excel.setDiscName("{" + excel.getDiscName() + "}");
//                }
//
//                if(StringUtils.isEmpty(excel.getRoomSelectionDate())){
//                    roomSelectionDateFlag = false;
//                    builder.append("选房日期为空").append(",");
//                    excel.setRoomSelectionDate("{" + excel.getRoomSelectionDate() + "}");
//                }else {
//                    try {
//                        Date date = DateUtils.parseDate(excel.getRoomSelectionDate(), "yyyy/MM/dd");
//                    }catch (Exception e){
//                        roomSelectionDateFlag = false;
//                        builder.append("选房日期格式有误").append(",");
//                        excel.setRoomSelectionDate("{" + excel.getRoomSelectionDate() + "}");
//                    }
//                }
//
//                if (StringUtils.isEmpty(excel.getBatchNumber())){
//                    batchNumberFlag = false;
//                    builder.append("批次号为空").append(",");
//                    excel.setBatchNumber("{" + excel.getBatchNumber() + "}");
//                }
//
//                if (StringUtils.isEmpty(excel.getRoomSelectionNumber())){
//                    roomSelectionNumberFlag = false;
//                    builder.append("选房号为空").append(",");
//                    excel.setRoomSelectionNumber("{" + excel.getRoomSelectionNumber() + "}");
//                }
//
//                if (StringUtils.isEmpty(excel.getBuyerName())){
//                    buyerNameFlag = false;
//                    builder.append("姓名为空").append(",");
//                    excel.setBuyerName("{" + excel.getBuyerName() + "}");
//                }
//
//                if (StringUtils.isEmpty(excel.getIdentityCard())){
//                    identityCardFlag = false;
//                    builder.append("身份证号为空").append(",");
//                    excel.setIdentityCard("{" + excel.getIdentityCard() + "}");
//                }else {
//                    if(!this.isIdentityCard(excel.getIdentityCard())){
//                        identityCardFlag = false;
//                        builder.append("身份证号格式有误").append(",");
//                        excel.setIdentityCard("{" + excel.getIdentityCard() + "}");
//                    }
//                }
//
//                if (StringUtils.isEmpty(excel.getPhone())){
//                    phoneFlag = false;
//                    builder.append("手机号为空").append(",");
//                    excel.setPhone("{" + excel.getPhone() + "}");
//                }else {
//                    if(!this.isPhone(excel.getPhone())){
//                        phoneFlag = false;
//                        builder.append("手机号格式有误").append(",");
//                        excel.setPhone("{" + excel.getPhone() + "}");
//                    }
//                }
//
//                // 关系
//                if (StringUtils.isNotEmpty(excel.getRelationship())) {
//                    for (Map<String, String> stringStringMap : relationship) {
//                        if (stringStringMap.get("label").equals(excel.getRelationship())) {
//                            excel.setRelationship(stringStringMap.get("value"));
//                            excel.setRelationshipStr(stringStringMap.get("label"));
//                        }
//                    }
//                    // 没匹配上的情况
//                    if (excel.getRelationship() == null) {
//                        relationshipFlag = false;
//                        builder.append("关系格式有误").append(",");
//                        excel.setRelationship("{" + excel.getRelationship() + "}");
//                    }
//                }
//
//
//                if (StringUtils.isEmpty(excel.getOrderNum())){
//                    relationshipFlag = false;
//                    builder.append("顺序号为空").append(",");
//                    excel.setOrderNum("{" + excel.getOrderNum() + "}");
//                }
//
//                if (groupNameFlag && discNameFlag && roomSelectionDateFlag && batchNumberFlag && roomSelectionNumberFlag
//                        && buyerNameFlag && identityCardFlag && phoneFlag && relationshipFlag){
//
//                }else {
//                    throw new RuntimeException(builder.toString());
//                }
//
//            }catch (Exception e){
//                String message = e.getMessage();
//                excel.setReason(message);
//            }
//        }
//
//        // 合并新增
//        Map<String, List<CustomerManagementExcel>> map = new HashMap<>();
//        for(CustomerManagementExcel excel : list){
//            // 判断批次号+选房号是否重复
//            String key = excel.getBatchNumber() + "-" + excel.getRoomSelectionNumber();
//            if(map.containsKey(key)){
//                // 批次号+选房号重复 合并选房人信息
//                List<CustomerManagementExcel> oldExcels = map.get(key);
//                oldExcels.add(excel);
//                map.put(key, oldExcels);
//            }else {
//                List<CustomerManagementExcel> excelList = new ArrayList<>();
//                excelList.add(excel);
//                map.put(key, excelList);
//            }
//        }
//
//        // 合并后数据
//        for (Map.Entry<String, List<CustomerManagementExcel>> entry : map.entrySet()) {
//            // 批次号+选房号 对应的数据
//            List<CustomerManagementExcel> value = entry.getValue();
//            CustomerManagement customerManagement = new CustomerManagement();
//            BeanUtil.copyProperties(value.get(0), customerManagement);
//            String id = IdUtil.simpleUUID();
//            List<Buyer> buyers = new ArrayList<>();
//            // 拼接买受人姓名
//            StringBuilder builder = new StringBuilder();
//            boolean insertFlag = true;
//            for (CustomerManagementExcel excel : value){
//                if(StringUtils.isEmpty(excel.getReason())){
//                    Buyer buyer = new Buyer();
//                    buyer.setCustomerManagementId(id);
//                    buyer.setCreatedAt(new Date());
//                    buyer.setCreatedId(userId);
//                    buyers.add(buyer);
//                    if(StringUtils.isEmpty(excel.getRelationship())){
//                        builder.append(excel.getBuyerName()).append(",");
//                    }else {
//                        builder.append(excel.getBuyerName())
//                                .append("(")
//                                .append(excel.getRelationshipStr())
//                                .append(")")
//                                .append(",");
//                    }
//                    if(builder.length() > 0){
//                        builder.deleteCharAt(builder.length()-1);
//                    }
//
//                }else {
//                    insertFlag = false;
//                    badList.add(excel);
//                }
//            }
//            if(insertFlag){
//                customerManagement.setBuyerName(builder.toString());
//                customerManagement.setId(id);
//                customerManagement.setStatus(1);
//                customerManagement.setGroupId(parkNameIdMap.get(value.get(0).getGroupName()));
//                customerManagement.setDiscId(discNameIdMap.get(value.get(0).getDiscName()));
//                customerManagement.setCreatedAt(new Date());
//                customerManagement.setCreatedId(userId);
//                modelMapper.insertSelective(customerManagement);
//                for (Buyer buyer : buyers){
//                    buyer.setCustomerManagementId(id);
//                    buyer.setCreatedAt(new Date());
//                    buyer.setCreatedId(userId);
//                    buyerService.insertSelective(buyer);
//                }
//            }
//        }
//
//        count = list.size();
//        jsonObject.put("successCount", count - badList.size());
//        jsonObject.put("errorList", badList);
//        jsonObject.put("errorCount", badList.size());
//        list.clear();
//        return jsonObject;
//    }


    public JSONObject handleImport(MultipartFile file) throws IOException {

        String userId = ShiroUtils.getUserId();
        Date createdAt = new Date();
        // 读取excel
        ExcelUtils excelUtils = new ExcelUtils();
        List<CustomerManagementExcel> list = excelUtils.excelImport(file.getInputStream(), CustomerManagementExcel.class,2);

        List<CustomerManagementExcel> badList = new ArrayList<>();
        int count;
        JSONObject jsonObject = new JSONObject();

        Map<String, String> numberIdMap = getNumberIdMap();


        if (CollectionUtils.isNotEmpty(list)) {
            // 先循环判断表格中是否有重复数据
            Map<String, Integer> map = new HashMap<>();
            for (CustomerManagementExcel excelItem : list) {
                map.put(excelItem.getBatchNumber() + "-" + excelItem.getRoomSelectionNumber(), map.getOrDefault(excelItem.getBatchNumber() + "-" + excelItem.getRoomSelectionNumber(), 0) + 1);
            }
            for (CustomerManagementExcel excelItem : list) {
                if (map.get(excelItem.getBatchNumber() + "-" + excelItem.getRoomSelectionNumber()) > 1) {
                    excelItem.setDuplicate(true);
                } else {
                    excelItem.setDuplicate(false);
                }
            }

            CustomerManagement data;
            Buyer buyer;

            // 先校验
            for (CustomerManagementExcel excel : list) {

                data = new CustomerManagement();

                boolean roomSelectionDateFlag = true;
                boolean batchNumberFlag = true;
                boolean roomSelectionNumberFlag = true;
                boolean buyerNameFlag = true;
                boolean identityCardFlag = true;
                boolean phoneFlag = true;
                boolean duplicateFlag = true;

                StringBuilder builder = new StringBuilder();
                try {

                    // 重复数据标识
                    if (excel.getDuplicate()) {
                        duplicateFlag = false;
                        Class<?> clazz = excel.getClass();
                        Field[] fields = clazz.getDeclaredFields();
                        // 给所有字段打上标记 标红
                        for (Field field : fields) {
                            field.setAccessible(true);
                            if(field.getType().equals(String.class) && field.get(excel) != null){
                                field.set(excel, "{" + field.get(excel) + "}");
                            }
                        }
                        builder.append("数据重复");
                        throw new RuntimeException(builder.toString());
                    }else {
                        // 判断库中是否存在
                        if (numberIdMap.get(excel.getBatchNumber() + "-" + excel.getRoomSelectionNumber()) != null) {
                            Class<?> clazz = excel.getClass();
                            Field[] fields = clazz.getDeclaredFields();
                            // 给所有字段打上标记 标红
                            for (Field field : fields) {
                                field.setAccessible(true);
                                if(field.getType().equals(String.class) && field.get(excel) != null){
                                    field.set(excel, "{" + field.get(excel) + "}");
                                }
                            }
                            builder.append("系统中已存在,不可重复导入");
                            throw new RuntimeException(builder.toString());
                        }

                        // 选房日期
//                        if (StringUtils.isEmpty(excel.getRoomSelectionDate())) {
//                            roomSelectionDateFlag = false;
//                            builder.append("选房日期为空").append(",");
//                            excel.setRoomSelectionDate("{" + excel.getRoomSelectionDate() + "}");
//                        } else {
//                            try {
//                                Date roomSelectionDate = DateUtils.parseDate(excel.getRoomSelectionDate(), "yyyy/MM/dd HH:mm");
//                                data.setRoomSelectionDate(roomSelectionDate);
//                            } catch (Exception e) {
//                                roomSelectionDateFlag = false;
//                                builder.append("选房日期格式有误").append(",");
//                                excel.setRoomSelectionDate("{" + excel.getRoomSelectionDate() + "}");
//                            }
//                        }
                        if(StringUtils.isNotEmpty(excel.getRoomSelectionDate())){
                            try {
                                Date roomSelectionDate = DateUtils.parseDate(excel.getRoomSelectionDate(), "yyyy/MM/dd HH:mm");
                                data.setRoomSelectionDate(roomSelectionDate);
                            } catch (Exception e) {
                                roomSelectionDateFlag = false;
                                builder.append("选房日期格式有误").append(",");
                                excel.setRoomSelectionDate("{" + excel.getRoomSelectionDate() + "}");
                            }
                        }

                        // 批次号
                        if (StringUtils.isEmpty(excel.getBatchNumber())) {
                            batchNumberFlag = false;
                            builder.append("批次号为空").append(",");
                            excel.setBatchNumber("{" + excel.getBatchNumber() + "}");
                        }
                        data.setBatchNumber(excel.getBatchNumber());

                        // 选房号
                        if (StringUtils.isEmpty(excel.getRoomSelectionNumber())) {
                            roomSelectionNumberFlag = false;
                            builder.append("选房号为空").append(",");
                            excel.setRoomSelectionNumber("{" + excel.getRoomSelectionNumber() + "}");
                        }
                        data.setRoomSelectionNumber(excel.getRoomSelectionNumber());

                        // 购房1-姓名
                        if (StringUtils.isEmpty(excel.getBuyerName())) {
                            buyerNameFlag = false;
                            builder.append("姓名为空").append(",");
                            excel.setBuyerName("{" + excel.getBuyerName() + "}");
                        }
                        data.setBuyerName(excel.getBuyerName());

                        // 购房1-身份证号
                        if (StringUtils.isEmpty(excel.getIdentityCard())) {
                            identityCardFlag = false;
                            builder.append("身份证号为空").append(",");
                            excel.setIdentityCard("{" + excel.getIdentityCard() + "}");
                        } else {
                            if (!this.isIdentityCard(excel.getIdentityCard())) {
                                identityCardFlag = false;
                                builder.append("身份证号格式有误").append(",");
                                excel.setIdentityCard("{" + excel.getIdentityCard() + "}");
                            }
                        }
                        data.setBuyerIdentityCard(excel.getIdentityCard());

                        // 购房1-手机号
                        if (StringUtils.isEmpty(excel.getPhone())) {
                            phoneFlag = false;
                            builder.append("手机号为空").append(",");
                            excel.setPhone("{" + excel.getPhone() + "}");
                        } else {
                            if (!this.isPhone(excel.getPhone())) {
                                phoneFlag = false;
                                builder.append("手机号格式有误").append(",");
                                excel.setPhone("{" + excel.getPhone() + "}");
                            }
                        }
                    }


                    if (roomSelectionDateFlag && batchNumberFlag && roomSelectionNumberFlag
                            && buyerNameFlag && identityCardFlag && phoneFlag && duplicateFlag) {

                        String customerManagementId = IdUtil.simpleUUID();
                        data.setId(customerManagementId);
                        data.setStatus(1);
                        data.setCreatedId(userId);
                        data.setCreatedAt(createdAt);
                        modelMapper.insertSelective(data);
                        buyer = new Buyer();
                        buyer.setCustomerManagementId(customerManagementId);
                        buyer.setName(excel.getBuyerName());
                        buyer.setIdentityCard(excel.getIdentityCard());
                        buyer.setPhone(excel.getPhone());
                        buyer.setOrderNum(1);
                        buyer.setCreatedId(userId);
                        buyer.setCreatedAt(createdAt);
                        buyerService.insertSelective(buyer);

                    } else {
                        throw new RuntimeException(builder.toString());
                    }

                } catch (Exception e) {
                    String message = e.getMessage();
                    excel.setReason("{" + message + "}");
                    badList.add(excel);
                }
            }

        }



        count = list.size();
        jsonObject.put("successCount", count - badList.size());
        jsonObject.put("errorList", badList);
        jsonObject.put("errorCount", badList.size());
        list.clear();
        return jsonObject;
    }


    /**
     * 校验身份证号
     *
     * @param idCard
     * @return
     */
    public boolean isIdentityCard(String idCard) {
        String idCardPattern = "^\\d{17}[0-9Xx]$";
        boolean isValid = idCard.matches(idCardPattern);
        return isValid;
    }

    /**
     * 校验手机号
     *
     * @param phone
     * @return
     */
    public boolean isPhone(String phone) {
        String idCardPattern = "^1[3-9]\\d{9}$";
        boolean isValid = phone.matches(idCardPattern);
        return isValid;
    }


    public void errorListExport(List<CustomerManagementExcel> list, HttpServletResponse response) throws IOException {


        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String date = sdf.format(new Date());
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        String fileName = URLEncoder.encode("失败记录" + date, "UTF-8");
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");

        String templateFileName = "/home/project/settleDown/tem/选房客户导出模板.xlsx";
        ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream())
                .registerWriteHandler(new WriteHandlerStrategy())
                .withTemplate(templateFileName)
                .build();
        WriteSheet writeSheet = EasyExcel.writerSheet()
                .build();
        FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.TRUE).build();

        // 填充数据
        excelWriter.fill(list, fillConfig, writeSheet);
        excelWriter.finish();

    }


    public void downLoadTemplate(HttpServletResponse response) throws IOException {

        File file = new File("/home/project/settleDown/tem/选房客户导入模板.xlsx");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String date = sdf.format(new Date());
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        String fileName = URLEncoder.encode("选房客户导入模板" + date, "UTF-8");
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
        OutputStream out = response.getOutputStream();
        IoUtil.copy(FileUtil.getInputStream(file), out);
        out.close();
    }


    public Map<String, String> getNumberIdMap() {
        Map<String, String> map = new HashMap<>();
        List<CustomerManagement> list = selectByExample(new CustomerManagementExample());
        if (CollectionUtils.isNotEmpty(list)) {
            for(CustomerManagement data:list){
                String number = data.getBatchNumber() + "-" + data.getRoomSelectionNumber();
                map.put(number, data.getId());
            }
        }
        return map;
    }


    public List<CustomerManagementDto> selectNeedSendList() {
        return extendMapper.selectNeedSendList();
    }

}
