package com.idea.message.service;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.message.mapper.MessageInfoExpandMapper;
import com.idea.message.mapper.MessageInfoMapper;
import com.idea.message.model.SysMessageInfo;
import com.idea.message.model.SysMessageInfoExample;
import com.idea.message.request.SysMessageRequest;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.support.Convert;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.util.SnowflakeIdWorker;
import com.rockstar.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service
public class MessageInfoService implements BaseService<SysMessageInfo, SysMessageInfoExample> {

    //文件mapper
    @Autowired
    private MessageInfoMapper modelMapper;
    @Autowired
    private MessageInfoExpandMapper expandMapper;


    @Override
    public int deleteByPrimaryKey(String ids) {
        List<String> lista = Convert.toListStrArray(ids);
        SysMessageInfoExample example = new SysMessageInfoExample();
        example.createCriteria().andIdIn(lista);
        return modelMapper.deleteByExample(example);
    }

    @Override
    public SysMessageInfo selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }
    @Override
    public int updateByPrimaryKeySelective(SysMessageInfo record) {
        if(StringUtils.isEmpty(record.getId())){
            record.setId(SnowflakeIdWorker.getUUID());
            record.setCreateTime(new Date());
            return modelMapper.insert(record);
        }
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(SysMessageInfo record, SysMessageInfoExample example) {
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(SysMessageInfo record, SysMessageInfoExample example) {
        return modelMapper.updateByExample(record, example);
    }

    @Override
    public List<SysMessageInfo> selectByExample(SysMessageInfoExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(SysMessageInfoExample example) {
        return modelMapper.countByExample(example);
    }

    @Override
    public int deleteByExample(SysMessageInfoExample example) {
        return modelMapper.deleteByExample(example);
    }


    public int insertWithoutId(SysMessageInfo record){
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(SysMessageInfo record) {
        record.setId(SnowflakeIdWorker.getUUID());
        return modelMapper.insertSelective(record);
    }

    public int insert(SysMessageInfo record){
        record.setId(SnowflakeIdWorker.getUUID());
        return modelMapper.insert(record);
    }

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<SysMessageInfo> list(Tablepar tablepar, SysMessageInfo model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<SysMessageInfo> list = modelMapper.selectByExample(getCondition(model,dt));
        PageInfo<SysMessageInfo> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }
    /**
     * 查询指定
     *
     * @return
     */
    public List<SysMessageInfo> listAll(SysMessageInfo model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    public SysMessageInfo getSysMessageInfo(SysMessageInfo model){
        SysMessageInfo SysMessageInfo = new SysMessageInfo();
        List<SysMessageInfo> SysMessageInfos = modelMapper.selectByExample(getCondition(model, null));
        if(SysMessageInfos!=null&&SysMessageInfos.size()>0){
            SysMessageInfo = SysMessageInfos.get(0);
        }
        return SysMessageInfo;
    }

    private SysMessageInfoExample getCondition(SysMessageInfo model, DateTrans dt) {
        SysMessageInfoExample ex = new SysMessageInfoExample();
        if(null == model){
            return ex;
        }
        ex.setOrderByClause("create_time desc");
        SysMessageInfoExample.Criteria criteria = ex.createCriteria();


        return ex;
    }

    public PageInfo<SysMessageInfo> findByList(Tablepar tablepar, SysMessageRequest model) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<SysMessageInfo> list = expandMapper.selectByList(model);
        PageInfo<SysMessageInfo> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    // 已读
    public PageInfo<SysMessageInfo> getInReadList(Tablepar tablepar) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<SysMessageInfo> list = expandMapper.selectByInRead(ShiroUtils.getUserId());
        PageInfo<SysMessageInfo> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    // 未读
    public PageInfo<SysMessageInfo> getNotReadList(Tablepar tablepar) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<SysMessageInfo> list = expandMapper.selectByNotRead(ShiroUtils.getUserId());
        PageInfo<SysMessageInfo> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    // 获取版本号
    public SysMessageInfo getSysVersionNo(){
        return expandMapper.getSysVersionNo();
    }
}
