package com.idea.message.service;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.message.mapper.MessageReadMapper;
import com.idea.message.model.SysMessageRead;
import com.idea.message.model.SysMessageReadExample;
import com.rockstar.biz.model.BizUser;
import com.rockstar.biz.model.BizUserExample;
import com.rockstar.biz.service.BizUserService;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.support.Convert;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FramePermissionService;
import com.rockstar.util.SnowflakeIdWorker;
import com.rockstar.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service
public class MessageReadService implements BaseService<SysMessageRead, SysMessageReadExample> {

    //文件mapper
    @Autowired
    private MessageReadMapper modelMapper;

    @Autowired
    private FramePermissionService framePermissionService;

    @Autowired
    private BizUserService bizUserService;

    @Override
    public int deleteByPrimaryKey(String ids) {
        List<String> lista = Convert.toListStrArray(ids);
        SysMessageReadExample example = new SysMessageReadExample();
        example.createCriteria().andIdIn(lista);
        return modelMapper.deleteByExample(example);
    }

    @Override
    public SysMessageRead selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }
    @Override
    public int updateByPrimaryKeySelective(SysMessageRead record) {
        if(StringUtils.isEmpty(record.getId())){
            record.setId(SnowflakeIdWorker.getUUID());
            record.setReadTime(new Date());
            return modelMapper.insert(record);
        }
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(SysMessageRead record, SysMessageReadExample example) {
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(SysMessageRead record, SysMessageReadExample example) {
        return modelMapper.updateByExample(record, example);
    }

    @Override
    public List<SysMessageRead> selectByExample(SysMessageReadExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(SysMessageReadExample example) {
        return modelMapper.countByExample(example);
    }

    @Override
    public int deleteByExample(SysMessageReadExample example) {
        return modelMapper.deleteByExample(example);
    }


    public int insertWithoutId(SysMessageRead record){
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(SysMessageRead record) {
        record.setId(SnowflakeIdWorker.getUUID());
        return modelMapper.insertSelective(record);
    }

    public int insert(SysMessageRead record){
        record.setId(SnowflakeIdWorker.getUUID());
        return modelMapper.insert(record);
    }

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<SysMessageRead> list(Tablepar tablepar, SysMessageRead model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<SysMessageRead> list = modelMapper.selectByExample(getCondition(model,dt));
        PageInfo<SysMessageRead> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }
    /**
     * 查询指定
     *
     * @return
     */
    public List<SysMessageRead> listAll(SysMessageRead model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    public SysMessageRead getSysMessageRead(SysMessageRead model){
        SysMessageRead SysMessageRead = new SysMessageRead();
        List<SysMessageRead> SysMessageReads = modelMapper.selectByExample(getCondition(model, null));
        if(SysMessageReads!=null&&SysMessageReads.size()>0){
            SysMessageRead = SysMessageReads.get(0);
        }
        return SysMessageRead;
    }

    private SysMessageReadExample getCondition(SysMessageRead model, DateTrans dt) {
        SysMessageReadExample ex = new SysMessageReadExample();
        if(null == model){
            return ex;
        }
        ex.setOrderByClause("created_at desc");
        SysMessageReadExample.Criteria criteria = ex.createCriteria();
        if(StringUtils.isNotEmpty(model.getUserId())){
            criteria.andUserIdEqualTo(model.getUserId());
        }
        return ex;
    }

    public List<BizUser> bizUserListAll(String mainId){
        BizUserExample example = new BizUserExample();
        example.createCriteria().andMainIdEqualTo(mainId);
        return bizUserService.selectByExample(example);
    }

}
