package com.idea.paymentManagement.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.mapper.ContractManageMapper;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.mapper.RoomSelectionInfoMapper;
import com.idea.customerManagement.model.*;
import com.idea.customerManagement.service.BuyerService;
import com.idea.paymentManagement.dto.RefundManageDto;
import com.idea.paymentManagement.mapper.PayLogMapper;
import com.idea.paymentManagement.mapper.RefundManageExtendMapper;
import com.idea.paymentManagement.mapper.RefundManageMapper;
import com.idea.paymentManagement.model.*;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.mapper.FrameUserMapper;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.TableSplitResult;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class RefundManageService implements BaseService<RefundManage, RefundManageExample> {

    @Autowired
    private RefundManageMapper modelMapper;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private RoomSelectionInfoMapper roomSelectionInfoMapper;
    @Autowired
    private ParkRoomExtendMapper roomExtendMapper;
    @Autowired
    private RefundDetailService refundDetailService;
    @Autowired
    private RefundManageExtendMapper extendMapper;
    @Autowired
    private ContractManageMapper contractManageMapper;
    @Autowired
    private FrameUserMapper frameUserMapper;
    @Autowired
    private CustomerManagementMapper customerManagementMapper;
    @Autowired
    private ParkRoomMapper roomMapper;
    @Autowired
    private PayLogMapper payLogMapper;


    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(RefundManage refundManage) {
        refundManage.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(refundManage);
    }

    public int insertWithoutId(RefundManage refundManage) {
        return modelMapper.insertSelective(refundManage);
    }

    @Override
    public RefundManage selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(RefundManage refundManage) {
        return modelMapper.updateByPrimaryKeySelective(refundManage);
    }

    @Override
    public int updateByExampleSelective(RefundManage refundManage, RefundManageExample refundManageExample) {
        return modelMapper.updateByExampleSelective(refundManage, refundManageExample);
    }

    @Override
    public int updateByExample(RefundManage refundManage, RefundManageExample refundManageExample) {
        return modelMapper.updateByExample(refundManage, refundManageExample);
    }

    @Override
    public List<RefundManage> selectByExample(RefundManageExample refundManageExample) {
        return modelMapper.selectByExample(refundManageExample);
    }

    @Override
    public long countByExample(RefundManageExample refundManageExample) {
        return modelMapper.countByExample(refundManageExample);
    }

    @Override
    public int deleteByExample(RefundManageExample refundManageExample) {
        return modelMapper.deleteByExample(refundManageExample);
    }


    /**
     * 根据身份证号带出关联房屋
     *
     * @param identityCard
     * @return
     */
    public List<ParkRoomResponse> getHouseListByIdCard(String identityCard) {

        // 根据身份证查询 买方信息
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.createCriteria().andIdentityCardEqualTo(identityCard);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        List<String> customerManagementIds = buyers.stream().map(Buyer::getCustomerManagementId).collect(Collectors.toList());
        // 根据客户管理信息 查询选房信息 剔除退房的
        RoomSelectionInfoExample roomSelectionInfoExample = new RoomSelectionInfoExample();
        roomSelectionInfoExample.createCriteria().andCustomerManagementIdIn(customerManagementIds)
                .andStatusNotEqualTo(5);
        List<RoomSelectionInfo> selectionList = roomSelectionInfoMapper.selectByExample(roomSelectionInfoExample);
        List<ParkRoomResponse> result = new ArrayList<>();
        for (RoomSelectionInfo info : selectionList) {
            // 根据客户id 合同房间id 查询合同
            ContractManageExample example = new ContractManageExample();
            example.createCriteria().andCustomerManagementIdEqualTo(info.getCustomerManagementId())
                    .andHouseIdEqualTo(info.getHouseId());
            List<ContractManage> contractManages = contractManageMapper.selectByExample(example);

            ParkRoomResponse response = roomExtendMapper.getFullName(info.getHouseId());
            response.setCustomerManagementId(info.getCustomerManagementId());
            response.setContractId(contractManages.get(0).getId());
            result.add(response);
        }
        return result;
    }


    /**
     * 新增退款
     *
     * @param model
     * @return
     */
    @Transactional
    public int add(RefundManageDto model) {

        ContractManage contractManage = contractManageMapper.selectByPrimaryKey(model.getContractId());
        // 已退房
        if (contractManage.getContractStatus() == 3) {
            throw new RuntimeException("该房间已退款,请重新选择!!!");
        }

        String id = IdUtil.simpleUUID();
        String userId = ShiroUtils.getUserId();
        Date date = new Date();
        List<RefundDetail> details = JSONObject.parseArray(model.getRefundDetailInfoStr(), RefundDetail.class);
        Integer collectionStatus = null;
        Integer fundCollectionStatus = null;
        for (RefundDetail detail : details) {
            if (detail.getType().equals("定金") || detail.getType().equals("房款")) {
                collectionStatus = 3;
            }
            if (detail.getType().equals("专项维修资金")) {
                fundCollectionStatus = 3;
            }
            detail.setRefundManageId(id);
            detail.setCreatedId(userId);
            detail.setCreatedAt(date);
            refundDetailService.insertSelective(detail);
            // 收款记录 更新退款状态
            String[] split = detail.getPayLogId().split(",");
            for (String paylogId : split) {
                PayLog payLog = payLogMapper.selectByPrimaryKey(paylogId);
                payLog.setPaymentStatus(2);
                payLogMapper.updateByPrimaryKeySelective(payLog);
            }


        }
        model.setId(id);
        model.setCreatedId(userId);
        model.setCreatedAt(date);
        int result = insertWithoutId(model);

        // 更新合同 收款状态 合同状态
        contractManage.setCollectionStatus(collectionStatus);
        contractManage.setFundCollectionStatus(fundCollectionStatus);
        contractManage.setContractStatus(3);
        contractManageMapper.updateByPrimaryKeySelective(contractManage);
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(contractManage.getCustomerManagementId());
        // 更新客户管理 客户状态
        customerManagement.setStatus(5);
        customerManagementMapper.updateByPrimaryKeySelective(customerManagement);
        // 更新房间状态
        ParkRoom room = roomMapper.selectByPrimaryKey(contractManage.getHouseId());
        room.setSoldStatus(4);
        room.setIsChoose(0);
        roomMapper.updateByPrimaryKeySelective(room);
        // 更新定金管理 收取状态
        RoomSelectionInfoExample example = new RoomSelectionInfoExample();
        example.createCriteria().andCustomerManagementIdEqualTo(customerManagement.getId())
                .andHouseIdEqualTo(contractManage.getHouseId());
        List<RoomSelectionInfo> list = roomSelectionInfoMapper.selectByExample(example);
        RoomSelectionInfo roomSelectionInfo = list.get(0);
        roomSelectionInfo.setStatus(5);
        roomSelectionInfoMapper.updateByPrimaryKeySelective(roomSelectionInfo);

        return result;
    }

    /**
     * 编辑退款
     *
     * @param model
     * @return
     */
    @Transactional
    public int edit(RefundManageDto model) {

        String id = model.getId();
        String userId = ShiroUtils.getUserId();
        Date date = new Date();
        // 先删除
        RefundDetailExample refundDetailExample = new RefundDetailExample();
        refundDetailExample.createCriteria().andRefundManageIdEqualTo(model.getId());
        List<RefundDetail> oldDetails = refundDetailService.selectByExample(refundDetailExample);
        for (RefundDetail detail : oldDetails) {
            refundDetailService.deleteByPrimaryKey(detail.getId());
        }
        // 再新增
        List<RefundDetail> details = JSONObject.parseArray(model.getRefundDetailInfoStr(), RefundDetail.class);
        for (RefundDetail detail : details) {
            detail.setRefundManageId(id);
            detail.setCreatedId(userId);
            detail.setCreatedAt(date);
            refundDetailService.insertSelective(detail);
        }
        model.setId(id);
        model.setUpdatedId(userId);
        model.setUpdatedAt(date);
        int result = updateByPrimaryKeySelective(model);
        return result;
    }

    /**
     * 退款列表
     *
     * @param tablepar
     * @param model
     * @param dt
     * @return
     */
    public PageInfo<RefundManageDto> listByModel(Tablepar tablepar, RefundManageDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<RefundManageDto> list = extendMapper.listByModel(model);
        PageInfo<RefundManageDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    public List<RefundManageDto> listAll(RefundManageDto model) {
        List<RefundManageDto> list = extendMapper.listByModel(model);
        return list;
    }


    public RefundManageDto getById(String id) {
        RefundManage refundManage = selectByPrimaryKey(id);
        RefundManageDto result = new RefundManageDto();
        FrameUser frameUser = frameUserMapper.selectByPrimaryKey(refundManage.getCreatedId());
        String userName = "";
        userName = frameUser == null ? "" : frameUser.getTruename();
        result.setCreatedName(userName);
        BeanUtil.copyProperties(refundManage, result);
        return result;
    }


}
