package com.idea.workPanel.controller;

import com.github.pagehelper.PageInfo;
import com.idea.workPanel.enume.WorkTrendsTypeEnum;
import com.idea.workPanel.model.ParkWaitHandle;
import com.idea.workPanel.model.ParkWaitRead;
import com.idea.workPanel.model.ParkWorkTrends;
import com.idea.workPanel.service.ParkWaitHandleService;
import com.idea.workPanel.service.ParkWaitReadService;
import com.idea.workPanel.service.ParkWorkTrendsService;
import com.idea.workPanel.vo.ParkWorkCountVo;
import com.rockstar.common.base.BaseController;
import com.rockstar.frame.model.extend.TableSplitResult;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.util.StringUtils;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Controller
@RequestMapping(value = "WorkPanelController")
@Api(value="超级工作台")
public class WorkPanelController extends BaseController {
    @Autowired
    private ParkWaitHandleService parkWaitHandleService;
    @Autowired
    private ParkWaitReadService parkWaitReadService;
    @Autowired
    private ParkWorkTrendsService parkWorkTrendsService;

    // 获取我的待办
    @PostMapping(value = "handleList",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object handleList(Integer type, Tablepar tablepar){
        PageInfo<ParkWaitHandle> page = parkWaitHandleService.getMyHandleList(tablepar, type);
        TableSplitResult<ParkWaitHandle> result=new TableSplitResult<ParkWaitHandle>(page.getPageNum(), page.getTotal(), page.getList());
        return result;
    }

    // 获取待办各类型数量
    @PostMapping(value = "getHandleSize",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object getHandleSize() {
        String userId = ShiroUtils.getUserId();
        List<ParkWorkCountVo> voList = new ArrayList<>();
        for (int i = 1; i < 8; i++) {
            Integer size = parkWaitHandleService.getMyHandleSize(userId, i);
            if(size > 0)
                voList.add(new ParkWorkCountVo(i, size, 0));
        }
        return success(voList);
    }

    // 获取我的待阅
    @PostMapping(value = "readList",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object readList(Integer type, Tablepar tablepar){
        PageInfo<ParkWaitRead> page = parkWaitReadService.getMyReadList(tablepar, type);
        TableSplitResult<ParkWaitRead> result=new TableSplitResult<ParkWaitRead>(page.getPageNum(), page.getTotal(), page.getList());
        return result;
    }

    // 获取待阅各类型数量
    @PostMapping(value = "getReadSize",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object getReadSize() {
        String userId = ShiroUtils.getUserId();
        List<ParkWorkCountVo> voList = new ArrayList<>();
        List<Integer> list = Arrays.asList(1, 2, 3, 7, 8);
        for(Integer num : list){
            Integer size = parkWaitReadService.getMyReadSize(userId, num);
            if(size > 0)
                voList.add(new ParkWorkCountVo(num, size, 1));
        }
//        for (int i = 1; i < 8; i++) {
//            Integer size = parkWaitReadService.getMyReadSize(userId, i);
//            if(size > 0)
//                voList.add(new ParkWorkCountVo(i, size, 1));
//        }
        return success(voList);
    }

    // 阅读
    @PostMapping(value = "userRead",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object userRead(String id) {
        parkWaitReadService.userRead(id);
        return success();
    }

    // 获取当前季度工作动态
    @PostMapping(value = "workTrendsList",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object workTrendsList(){
        List<ParkWorkTrends> trends = parkWorkTrendsService.getQuarterList();
        trends.stream().forEach(item -> {
            if(StringUtils.isNotEmpty(item.getWorkType())){
                WorkTrendsTypeEnum en = WorkTrendsTypeEnum.getNameByValue(item.getWorkType());
                if(null != en){
                    item.setWorkType(en.getName());
                }
            }
        });
        return success(trends);
    }
}
