package com.idea.workPanel.service;

import cn.hutool.core.collection.CollUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.workPanel.enume.ReadTypeEnum;
import com.idea.workPanel.mapper.ParkWaitReadMapper;
import com.idea.workPanel.mapper.ParkWaitReadOnMapper;
import com.idea.workPanel.model.ParkWaitRead;
import com.idea.workPanel.model.ParkWaitReadExample;
import com.idea.workPanel.model.ParkWaitReadOn;
import com.idea.workPanel.model.ParkWaitReadOnExample;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.support.Convert;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.util.SnowflakeIdWorker;
import com.rockstar.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ParkWaitReadService implements BaseService<ParkWaitRead, ParkWaitReadExample> {
    @Autowired
    private FrameUserService frameUserService;
    @Autowired
    private ParkWaitReadMapper modelMapper;
    @Autowired
    private ParkWaitReadOnMapper modelOnMapper;


    @Override
    public int deleteByPrimaryKey(String ids) {
        List<String> lista = Convert.toListStrArray(ids);
        ParkWaitReadExample example = new ParkWaitReadExample();
        example.createCriteria().andIdIn(lista);
        return modelMapper.deleteByExample(example);
    }

    @Override
    public ParkWaitRead selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }
    @Override
    public int updateByPrimaryKeySelective(ParkWaitRead record) {
        if(StringUtils.isEmpty(record.getId())){
            record.setId(SnowflakeIdWorker.getUUID());
            record.setCreatedAt(new Date());
            return modelMapper.insert(record);
        }
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(ParkWaitRead record, ParkWaitReadExample example) {
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(ParkWaitRead record, ParkWaitReadExample example) {
        return modelMapper.updateByExample(record, example);
    }

    @Override
    public List<ParkWaitRead> selectByExample(ParkWaitReadExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(ParkWaitReadExample example) {
        return modelMapper.countByExample(example);
    }

    @Override
    public int deleteByExample(ParkWaitReadExample example) {
        return modelMapper.deleteByExample(example);
    }


    public int insertWithoutId(ParkWaitRead record){
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(ParkWaitRead record) {
        record.setId(SnowflakeIdWorker.getUUID());
        return modelMapper.insertSelective(record);
    }

    public int insert(ParkWaitRead record){
        record.setId(SnowflakeIdWorker.getUUID());
        return modelMapper.insert(record);
    }

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<ParkWaitRead> list(Tablepar tablepar, ParkWaitRead model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ParkWaitRead> list = modelMapper.selectByExample(getCondition(model,dt));
        PageInfo<ParkWaitRead> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }
    /**
     * 查询指定
     *
     * @return
     */
    public List<ParkWaitRead> listAll(ParkWaitRead model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    public ParkWaitRead getParkWaitRead(ParkWaitRead model){
        ParkWaitRead ParkWaitRead = new ParkWaitRead();
        List<ParkWaitRead> ParkWaitReads = modelMapper.selectByExample(getCondition(model, null));
        if(ParkWaitReads!=null&&ParkWaitReads.size()>0){
            ParkWaitRead = ParkWaitReads.get(0);
        }
        return ParkWaitRead;
    }

    private ParkWaitReadExample getCondition(ParkWaitRead model, DateTrans dt) {
        ParkWaitReadExample ex = new ParkWaitReadExample();
        if(null == model){
            return ex;
        }
        ex.setOrderByClause("created_at desc");
        ParkWaitReadExample.Criteria criteria = ex.createCriteria();
        return ex;
    }

    // 获取待阅列表
    public PageInfo<ParkWaitRead> getMyReadList(Tablepar tablepar, Integer type) {
        ParkWaitReadOnExample onExample = new ParkWaitReadOnExample();
        onExample.createCriteria().andReadUserIdEqualTo(ShiroUtils.getUserId());
        List<ParkWaitReadOn> onList = modelOnMapper.selectByExample(onExample);
        List<String> onStr = onList.stream().map(ParkWaitReadOn::getReadId).collect(Collectors.toList());
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        ParkWaitReadExample example = new ParkWaitReadExample();
        example.setOrderByClause("created_at desc");
        ParkWaitReadExample.Criteria criteria = example.createCriteria();
        criteria.andReceiveUserIdEqualTo(ShiroUtils.getUserId());
        if(null != onStr && onStr.size() > 0)
            criteria.andIdNotIn(onStr);
        if(null != type)
            criteria.andReadTypeEqualTo(type);
        List<ParkWaitRead> list = modelMapper.selectByExample(example);
        PageInfo<ParkWaitRead> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    // 获取待阅列表
    public PageInfo<ParkWaitRead> getMyReadListMoblie(Tablepar tablepar, String types, String userId) {
        ParkWaitReadOnExample onExample = new ParkWaitReadOnExample();
        onExample.createCriteria().andReadUserIdEqualTo(userId);
        List<ParkWaitReadOn> onList = modelOnMapper.selectByExample(onExample);
        List<String> onStr = onList.stream().map(ParkWaitReadOn::getReadId).collect(Collectors.toList());
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        ParkWaitReadExample example = new ParkWaitReadExample();
        example.setOrderByClause("created_at desc");
        ParkWaitReadExample.Criteria criteria = example.createCriteria();
        if(StringUtils.isNotEmpty(userId)){
            FrameUser user = frameUserService.selectByPrimaryKey(userId);
            if(null != user){
                criteria.andReceiveUserIdIn(CollUtil.newArrayList("0", userId, user.getDataRole()));
            }
        }
        if(null != onStr && onStr.size() > 0)
            criteria.andIdNotIn(onStr);
        if(StringUtils.isNotEmpty(types)){
            List<Integer> typeList = Arrays.stream(types.split(",")).map(item -> Integer.parseInt(item)).collect(Collectors.toList());
            criteria.andReadTypeIn(typeList);
        }
        List<ParkWaitRead> list = modelMapper.selectByExample(example);
        PageInfo<ParkWaitRead> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    // 获取待阅数量
    public Integer getMyReadSize(String userId, Integer type) {
        ParkWaitReadOnExample onExample = new ParkWaitReadOnExample();
        onExample.createCriteria().andReadUserIdEqualTo(userId);
        List<ParkWaitReadOn> onList = modelOnMapper.selectByExample(onExample);
        List<String> onStr = onList.stream().map(ParkWaitReadOn::getReadId).collect(Collectors.toList());
        ParkWaitReadExample example = new ParkWaitReadExample();
        ParkWaitReadExample.Criteria criteria = example.createCriteria();
        criteria.andReceiveUserIdEqualTo(userId);
        if(null != onStr && onStr.size() > 0)
            criteria.andIdNotIn(onStr);
        if(null != type)
            criteria.andReadTypeEqualTo(type);
//        example.createCriteria().andIdNotIn(onStr).andReadTypeEqualTo(type);
        Long size = modelMapper.countByExample(example);
        return null==size?0:size.intValue();
    }

    // 阅读
    public void userRead(String id) {
        if(!StringUtils.isNotEmpty(id))
            return;
        ParkWaitReadOn on = new ParkWaitReadOn();
        on.setId(SnowflakeIdWorker.getUUID());
        on.setReadId(id);
        on.setReadUserId(ShiroUtils.getUserId());
        on.setReadTime(new Date());
        modelOnMapper.insert(on);
    }

    // 新增待阅

    /**
     * 新增待阅
     * @param en 枚举
     * @param title 标题
     * @param srcUserId 来源用户id
     * @param srcUserName 来源用户名称
     * @param srcUserType 来源用户类型 0管理,1企业
     * @param receiveUserId 接收用户id or 0=全部
     * @param routeParam 路由携带参数  例： id=***&type=**
     */
    public void addWaitReadey(ReadTypeEnum en, String title, String srcUserId, String srcUserName,
                              Integer srcUserType, String receiveUserId, String routeParam) {
        ParkWaitRead read = new ParkWaitRead();
        read.setReadType(en.getValue());
        read.setTitle(title);
        read.setSrcUserId(srcUserId);
        read.setSrcUserName(srcUserName);
        read.setSrcUserType(srcUserType);
        read.setReceiveUserId(receiveUserId);
        read.setRouteUrl(en.getUrl());
        read.setRouteParam(routeParam);
        try {
            read.setCreateBy(ShiroUtils.getUserId());
        } catch (Exception e) {
            read.setCreateBy("1");
        }
        read.setCreatedAt(new Date());
        insert(read);
//        //发送短信
//        FrameUser user = frameUserService.selectByPrimaryKey(receiveUserId);
//        if(StringUtils.isNotEmpty(user.getPhone())){
//            try {
//                SmsSendUtil.sendText(title,user.getPhone());
//            } catch (Exception e) {
//                log.error("短信发送异常:",e);
//                e.printStackTrace();
//            }
//        }

    }

}
