package com.idea.pro.wx.web;

import com.alibaba.fastjson.JSONArray;
import com.github.pagehelper.PageInfo;
import com.rockstar.common.base.BaseController;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.model.extend.TableSplitResult;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.system.model.SysDict;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

@Controller
@Api("字典管理")
@RequestMapping({"/wx/SysDictController"})
public class WxSysDictController extends BaseController {


    @Autowired
    private SysDictService modelService;


    @PostMapping(
            value = {"list"},
            produces = {"application/json;charset=UTF-8"}
    )
    @ResponseBody
    public Object list(Tablepar tablepar, SysDict model) {
        PageInfo<SysDict> page = this.modelService.list(tablepar, model);
        TableSplitResult<SysDict> result = new TableSplitResult(page.getPageNum(), page.getTotal(), page.getList());
        return result;
    }

    @PostMapping(
            value = {"checkCode/{code}"},
            produces = {"application/json;charset=UTF-8"}
    )
    @ResponseBody
    public boolean checkCode(@PathVariable("code") String code) {
        SysDict model = this.modelService.selectByDictCode(code);
        return null == model;
    }

    @PostMapping(
            value = {"add"},
            produces = {"application/json;charset=UTF-8"}
    )
    @ResponseBody
    public AjaxResult add(SysDict record) {
        int result = this.modelService.insert(record);
        return this.result(result);
    }

    @PostMapping(
            value = {"remove/{id}"},
            produces = {"application/json;charset=UTF-8"}
    )
    @ResponseBody
    public AjaxResult remove(@PathVariable("id") String id) {
        int result = this.modelService.deleteByPrimaryKey(id);
        return this.result(result);
    }

    @PostMapping(
            value = {"edit"},
            produces = {"application/json;charset=UTF-8"}
    )
    @ResponseBody
    public AjaxResult editSave(SysDict model) {
        int result = this.modelService.updateByPrimaryKeySelective(model);
        return this.result(result);
    }

    @PostMapping(
            value = {"getById"},
            produces = {"application/json;charset=UTF-8"}
    )
    @ResponseBody
    public SysDict getById(String id) {
        return this.modelService.selectByPrimaryKey(id);
    }

    @PostMapping(
            value = {"getByCode/{code}"},
            produces = {"application/json;charset=UTF-8"}
    )
    @ResponseBody
    public SysDict getByCode(@PathVariable("code") String code) {
        return this.modelService.selectByDictCode(code);
    }

    @PostMapping(
            value = {"getByCodes"},
            produces = {"application/json;charset=UTF-8"}
    )
    @ResponseBody
    public Map<String, SysDict> getByCodes(String codes) {
        Map<String, SysDict> dictDatas = new HashMap();
        if (StringUtils.isNotEmpty(codes)) {
            JSONArray arr = JSONArray.parseArray(codes);
            Iterator var4 = arr.iterator();

            while(var4.hasNext()) {
                Object code = var4.next();
                dictDatas.put(code.toString(), this.modelService.selectByDictCode(code.toString()));
            }
        }

        return dictDatas;
    }

    @PostMapping(
            value = {"selectDictMap"},
            produces = {"application/json;charset=UTF-8"}
    )
    @ResponseBody
    public Object selectDictMap(String code) {
        return this.success(this.modelService.selectDictMap(code));
    }

}
